/*

Copyright 2011 Danilo Ruffolo <danilo.ruffolo@gmail.com>

This file is part of nTetris.

nTetris is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

nTetris is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with nTetris.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include <ncurses.h>
#include <unistd.h>
#include <iostream>
#include <cstdlib>
#include <ctime>

#include "schermo.h"
#include "hiscore.h"
#include "tabellone.h"
#include "interfaccia.h"
#include "gioco.h"

#include "pezzoJ.h"
#include "pezzoT.h"
#include "pezzoS.h"
#include "pezzoI.h"
#include "pezzoO.h"
#include "pezzoZ.h"
#include "pezzoL.h"
 
using namespace std;

static PezzoJ j;
static PezzoT t;
static PezzoS s;
static PezzoI i;
static PezzoO o;
static PezzoZ z;
static PezzoL l;

int main()
{
    initscr();
    keypad(stdscr, true);
    raw();
    noecho();
    curs_set(0);
    srand(time(NULL));

    if(has_colors() == false)
    {
        endwin();
        cout << "Il terminale non supporta i colori." << endl
        << "Premere un tasto per uscire dal programma." << endl;
        getch();
        return 0;
    }

    if(LINES < 25 || COLS < 80)
    {
        printw("La dimensione del terminale è troppo piccola.\n");
        printw("Impostare la dimensione del terminale ad un minimo di 80x25 e riavviare il programma.\n\n");
        printw("Premere un tasto per uscire dal programma.");
        getch();
        endwin();
        return 0;
    }
        
    start_color();
    init_pair(1, COLOR_WHITE, COLOR_BLACK);
    init_pair(2, COLOR_BLUE, COLOR_BLACK);
    init_pair(3, COLOR_RED, COLOR_BLACK);
    init_pair(4, COLOR_GREEN, COLOR_BLACK);
    init_pair(5, COLOR_YELLOW, COLOR_BLACK);
    init_pair(6, COLOR_MAGENTA, COLOR_BLACK);
    init_pair(7, COLOR_CYAN, COLOR_BLACK);

    Tabellone tabellone;
    Interfaccia interfaccia;
    Gioco gioco;
    
    int menuPrincipale;
    int loop;
    int riga;
    int colonna;
    int pezzoInt;
    int formaInt;
    int prossimoPezzoInt;
    int prossimaFormaInt;
    Pezzo* pezzo;
    Pezzo* prossimoPezzo;
    
    while(true)
    {
        loop = 0;
        Schermo schermo;

        do
        {
            menuPrincipale = 0;
            clear();
            menuPrincipale = interfaccia.menuPrincipale(tabellone, schermo);
            switch(menuPrincipale)
            {
            case 0:
                break;
            case 1:
                return 0;
            }
        }
        while(menuPrincipale == 2);
        
        interfaccia.disegnaSchermo(schermo);
        interfaccia.refreshVariabili(schermo);
        
        nodelay(stdscr, true);

        pezzoInt = rand() % 7 + 1;
        formaInt = rand() % 4 + 1;
        
        while(loop != 2)
        {
            riga = 0;
            colonna = 4;
            prossimoPezzoInt = rand() % 7 + 1;
            prossimaFormaInt = rand() % 4 + 1;

            pezzo = schermo.generaPezzo(pezzoInt, j, t, s, i, o, z, l);
            prossimoPezzo = schermo.generaPezzo(prossimoPezzoInt, j, t, s, i, o, z, l);

            loop = gioco.loopGioco(interfaccia, schermo, tabellone, pezzo, prossimoPezzo, riga, colonna);
            
            if(loop == 1)
            {
                interfaccia.terminaProgramma(tabellone);
                return 0;
            }

            pezzo->setFormaAttuale(0);
            prossimoPezzo->setFormaAttuale(0);

            pezzoInt = prossimoPezzoInt;
            formaInt = prossimaFormaInt;

            interfaccia.refreshVariabili(schermo);
            
        }
    }
    
    getch();
    endwin();
    return 0;
}
